/**
 * MODULE PRESTASHOP OFFICIEL CHRONOPOST
 *
 * LICENSE : All rights reserved - COPY AND REDISTRIBUTION FORBIDDEN WITHOUT PRIOR CONSENT FROM OXILEO
 * LICENCE : Tous droits réservés, le droit d'auteur s'applique - COPIE ET REDISTRIBUTION INTERDITES
 * SANS ACCORD EXPRES D'OXILEO
 *
 * @author    Oxileo SAS <contact@oxileo.eu>
 * @copyright 2001-2018 Oxileo SAS
 * @license   Proprietary - no redistribution without authorization
 */

/* DECLARED IN .tpl
* cust_address,
* cust_city,
* cust_codePostal,
* cust_firstname,
* cust_lastname,
* chronodata,
* relais_map,
* map_markers
*/

/* GLOBAL VARIABLES */
var marker_group = null;
var defaultPostCode = "75001";

function capitalize(string)
{
    return string.charAt(0).toUpperCase() + string.slice(1);
}

/**
 * Submit postCode change and rebuild markers
 */
function postcodeChangeEvent() {

    // Assign postal code
    var newPostCode = $('#chronorelais_container #relais_codePostal').val();
    cust_address = newPostCode + ", France";
    oldCodePostal = cust_codePostal;
    cust_codePostal = newPostCode;

    // Init Leaflet map
    initRelaisMap(cust_address, cust_codePostal, cust_city, cust_country);
}

function toggleRelaisMap(cust_address, codePostal, city, e) {
    if ($("input.delivery_option_radio:checked").val() == CHRONORELAIS_ID + "," || $("input[name=id_carrier]:checked").val() == CHRONORELAIS_ID_INT + "," ||
        $("input.delivery_option_radio:checked").val() == RELAISEUROPE_ID + "," || $("input[name=id_carrier]:checked").val() == RELAISEUROPE_ID_INT + "," ||
        $("input.delivery_option_radio:checked").val() == RELAISDOM_ID + "," || $("input[name=id_carrier]:checked").val() == RELAISDOM_ID_INT + ",") {
        if (typeof e != "undefined") {
            e.stopPropagation();
        }
        // Show Chronorelais controls
        $('#chronorelais_container').show();
        $('#chrono_postcode_controls').hide();

        // show postcode controls only for true ChronoRelais
        if ($("input.delivery_option_radio:checked").val() == CHRONORELAIS_ID + "," || $("input[name=id_carrier]:checked").val() == CHRONORELAIS_ID_INT + ",") {
            $('#chrono_postcode_controls').show();
        }


        initRelaisMap(cust_address, codePostal, city, cust_country);

        // @TODO proper fix to handle the latlngbounds.extend()
        if ($("input.delivery_option_radio:checked").val() == RELAISDOM_ID + "," || $("input[name=id_carrier]:checked").val() == RELAISDOM_ID_INT + "," ||
            $("input.delivery_option_radio:checked").val() == RELAISEUROPE_ID + "," || $("input[name=id_carrier]:checked").val() == RELAISEUROPE_ID_INT + ","
        ) {
            setTimeout(function () {
                initRelaisMap(cust_address, codePostal, city, cust_country);
            }, 600)
        }

        return false;
    }

    // Hide Chronorelais controls
    $('#chronorelais_container').hide();
    $('#chrono_postcode_controls').hide();

}

/**
 * Initialize layers and build markers
 *
 * @param address
 * @param codePostal
 * @param city
 * @param country
 */
function initRelaisMap(address, codePostal, city, country) {
    // Get Leaflet map.
    if (relais_map === null) {
        relais_map = mapInit();
        // Initialize imagery
        L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
            attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
        }).addTo(relais_map);

        // Add markers layer
        marker_group = L.featureGroup();
        marker_group.addTo(relais_map);
    } else { // Clean old markers
        marker_group.clearLayers();
    }

    // Build all markers
    initRelaisMarkers(address, codePostal, city, country);
}

/**
 * Initialize Leaflet map
 *
 * @returns {*}
 */
function mapInit() {
    return L.map('chronorelais_map').setView([0, 0], 12);
}


function associateRelais(relaisID) {
    $.ajax({
        url: path + '/async/storePointRelais.php?relaisID=' + relaisID + '&cartID=' + cartID + '&customerFirstname=' + encodeURIComponent(cust_firstname) + '&customerLastname=' + encodeURIComponent(cust_lastname)
    });
}

/**
 * Create all "relais" list
 *
 * @param json
 */
function createAllPointRelais(json) {
    var htmlForTxtSection = '';
    var data = JSON.parse(json);

    if (data.errorCode !== 0) {
        alert('Code postal erroné, merci de modifier le code postal dans votre adresse de livraison.');
        // init with original zipcode
        cust_address = defaultPostCode + ", France";
        $('#relais_codePostal').val(defaultPostCode);
        oldCodePostal = null;
        initRelaisMap(cust_address, defaultPostCode);
        return;
    }

    if (data.hasOwnProperty("listePointRelais")) {
        var listePointsRelais = data.listePointRelais;
        if(listePointsRelais.length !== undefined) {
            for (var i = 0; i < listePointsRelais.length; i++) {

                // COMPAT WITH OLD RECHERCHEBT
                var id = listePointsRelais[i].identifiant;
                var nom = listePointsRelais[i].nom;
                var adresse = listePointsRelais[i].adresse1;
                var postalCode = listePointsRelais[i].codePostal;
                var localite = listePointsRelais[i].localite;

                // Create marker
                createRelaisMarker(listePointsRelais[i]);
                chronodata[id] = listePointsRelais[i];

                // Build corresponding "relais" input
                htmlForTxtSection += '<div class="checkbox"><label><input type="radio" name="chronorelaisSelect" id="bt' + id + '" value="' + id + '"';
                if (i === 0) {
                    htmlForTxtSection += " checked";
                }
                htmlForTxtSection += '/> ' + nom + ' - ' + adresse + ' - ' + postalCode + ' ' + localite + '</label></div>';
            }
            associateRelais(listePointsRelais[0].identifiant);
        } else {
            // COMPAT WITH OLD RECHERCHEBT
            var id = listePointsRelais.identifiant;
            var nom = listePointsRelais.nom;
            var adresse = listePointsRelais.adresse1;
            var postalCode = listePointsRelais.codePostal;
            var localite = listePointsRelais.localite;

            // Create marker
            createRelaisMarker(listePointsRelais);
            chronodata[id] = listePointsRelais;

            // Build corresponding "relais" input
            htmlForTxtSection += '<div class="checkbox"><label><input type="radio" name="chronorelaisSelect" id="bt' + id + '" value="' + id + '"';
            if (i === 0) {
                htmlForTxtSection += " checked";
            }
            htmlForTxtSection += '/> ' + nom + ' - ' + adresse + ' - ' + postalCode + ' ' + localite + '</label></div>';
            associateRelais(listePointsRelais.identifiant);
        }

        // Add all relais to list
        $('#chronorelais_container #relais_txt').html(htmlForTxtSection);

        // Listener for BT select in radio list
        $('input[name="chronorelaisSelect"]').change(function () {
            var relaisId = $('input[name="chronorelaisSelect"]:checked').val();
            associateRelais(relaisId);
            openMarker(relaisId);
        });

        // Open closest marker
        openMarker($('input[name="chronorelaisSelect"]').val());
        relais_map.fitBounds(marker_group.getBounds());
    }
}

/**
 * Open marker corresponding to given "relais" ID
 *
 * @param relaisId
 */
function openMarker(relaisId) {
    map_markers[relaisId].openPopup();
}

/**
 * Initialize all markers
 *
 * @param address
 * @param cp
 * @param city
 * @param country
 */
function initRelaisMarkers(address, cp, city, country) {
    if (city === null) {
        $.ajax({
            url: path + '/async/getPointRelais2.php?city=unknown&codePostal=' + cp + '&country=' + country,
            success: createAllPointRelais
        });
    } else {
        $.ajax({
            url: path + '/async/getPointRelais2.php?codePostal=' + cp + '&address=' + encodeURIComponent(address) + '&country=' + country + '&city=' + city,
            success: createAllPointRelais
        });
    }
}

/**
 * Create Leaflet Marker for given "relais"
 *
 * @param relais
 */
function createRelaisMarker(relais) {
    var relaisIcon = L.icon({
        iconUrl: path + '/views/img/postal.png',
        iconSize: [45, 30]
    });

    var marker = L.marker([relais.coordGeolocalisationLatitude, relais.coordGeolocalisationLongitude], {icon: relaisIcon});

    marker.on('click', function(e) {
        $("#bt"+relais.identifiant).prop("checked", true);
    });

    map_markers[relais.identifiant] = marker;
    marker.addTo(marker_group).bindPopup(buildMarkerPopup(relais));
}

/**
 * Build HTML Marker popup for given "relais"
 *
 * @param relais
 * @returns {string}
 */
function buildMarkerPopup(relais) {
    var popup = '<div class="pointRelais"><h4>'
        + relais.nom + '</h4><p>' + relais.adresse1 + '<br/ >' + relais.codePostal
        + ' ' + relais.localite + '</p><h5>' + trad_horaire_ouverture + '</h5><table><tbody>';

    for (var i = 0; i < relais.listeHoraireOuverture.length; i++) {
        var day = relais.listeHoraireOuverture[i];
        if (day.horairesAsString !== null) {
            popup += '<tr class="first_item item"><td>' + capitalize(days[day.jour - 1]) + '</td><td>'
                + day.horairesAsString + '</td></tr>';
        }
    }

    popup += '</tbody></table>'
        + '<p><input type="hidden" name="btID" value="' + relais.identifiant
        + '"/><a class="button_large btselect" href="javascript:;" class="pull-right">' + trad_selectionner + ' »</a></p>'
        + '</div>';

    return popup;
}

// Triggered on BT select from popup
function btSelect(target, e) {
    var btID = $(this).parent().children('input').val();
    var mObj = $('#relais_txt input[value=' + btID + ']');
    mObj.click();
    associateRelais(btID);
    $("html, body").animate({scrollTop: $('#relais_txt_cont').offset().top}, "slow");
}
